#!/usr/bin/env python
# -*- coding: utf-8 -*-
import podium_api
from podium_api.asyncreq import get_json_header, make_request_custom_success
from podium_api.types.token import get_token_from_json


def make_login_post(
    username, password, success_callback=None, failure_callback=None, progress_callback=None, redirect_callback=None
):
    """
    Request that hits the /oauth/token endpoint to log a user in. Will
    internally use **make_request_custom_success**.

    Args:
        username (string): The username to login

        password (string): The password for user.

    Kwargs:
        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(token (string))
        Defaults to None.

        failure_callback (function): Callback for redirects, failures, and
        errors. Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'redirect', 'failure'. Defaults
        to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """
    endpoint = "{}/oauth/token".format(podium_api.PODIUM_APP.podium_url)
    body = {"grant_type": "password", "username": username, "password": password}
    header = get_json_header()
    return make_request_custom_success(
        endpoint,
        login_success_handler,
        method="POST",
        success_callback=success_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        redirect_callback=redirect_callback,
        body=body,
        header=header,
    )


def login_success_handler(req, results, data):
    """
    Creates and returns a  PodiumToken to the success_callback found in data.

    Called automatically by **make_login_request**.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["success_callback"] is not None:
        data["success_callback"](get_token_from_json(results))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from podium_api.types.eventdevice import get_eventdevice_from_json

try:
    from urllib.parse import urlencode
except:
    from urllib import urlencode

import podium_api
from podium_api.asyncreq import get_json_header_token, make_request_custom_success
from podium_api.types.exceptions import NoEndpointOrIdsProvided
from podium_api.types.paged_response import get_paged_response_from_json
from podium_api.types.redirect import get_redirect_from_json


def make_eventdevices_get(
    token,
    event_id=None,
    endpoint=None,
    start=None,
    per_page=None,
    expand=True,
    quiet=None,
    success_callback=None,
    redirect_callback=None,
    failure_callback=None,
    progress_callback=None,
):
    """
    Request that returns a PodiumPagedRequest of event devices.
    By default a get request to
    'https://podium.live/api/v1/events/{event_id}/devices' will be made.

    Args:
        token (PodiumToken): The authentication token for this session.

    Kwargs:
        expand (bool): Expand all objects in response output. Defaults to True

        quiet (object): If not None HTML layout will not render endpoint
        description. Defaults to None.

        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(PodiumPagedResponse)
        Defaults to None.

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(result (dict), data (dict))
        Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

        start (int): Starting index for events list. 0 indexed.

        per_page (int): Number per page of results, max of 100.

        endpoint (str): If provided this endpoint will be used instead of
        the default: 'https://podium.live/api/v1/events/{event_id}/devices'

        event_id (int): If an endpoint is not provided you should provide
        the id of the event for which you want to look up the devices.

    Return:
        UrlRequest: The request being made.

    """
    if endpoint is None:
        if event_id is None:
            raise NoEndpointOrIdsProvided()
        endpoint = "{}/api/v1/events/{}/devices".format(podium_api.PODIUM_APP.podium_url, event_id)
    params = {}
    if expand is not None:
        params["expand"] = expand
    if quiet is not None:
        params["quiet"] = quiet
    if start is not None:
        params["start"] = start
    if per_page is not None:
        per_page = min(per_page, 100)
        params["per_page"] = per_page

    header = get_json_header_token(token)
    return make_request_custom_success(
        endpoint,
        eventdevices_success_handler,
        method="GET",
        success_callback=success_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        redirect_callback=redirect_callback,
        params=params,
        header=header,
    )


def make_livestreams_get(
    token,
    endpoint=None,
    start=None,
    per_page=None,
    expand=True,
    quiet=None,
    success_callback=None,
    redirect_callback=None,
    failure_callback=None,
    progress_callback=None,
):
    """
    Request that returns a PodiumPagedRequest of event devices for current livestreams.
    By default a get request to
    'https://podium.live/api/v1/livestreams' will be made.

    Args:
        token (PodiumToken): The authentication token for this session.

    Kwargs:
        expand (bool): Expand all objects in response output. Defaults to True

        quiet (object): If not None HTML layout will not render endpoint
        description. Defaults to None.

        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(PodiumPagedResponse)
        Defaults to None.

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(result (dict), data (dict))
        Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

        start (int): Starting index for events list. 0 indexed.

        per_page (int): Number per page of results, max of 100.

        endpoint (str): If provided this endpoint will be used instead of
        the default: 'https://podium.live/api/v1/eventdevices'

    Return:
        UrlRequest: The request being made.

    """
    if endpoint is None:
        endpoint = "{}/api/v1/livestreams".format(podium_api.PODIUM_APP.podium_url)
    params = {}
    if expand is not None:
        params["expand"] = expand
    if quiet is not None:
        params["quiet"] = quiet
    if start is not None:
        params["start"] = start
    if per_page is not None:
        per_page = min(per_page, 100)
        params["per_page"] = per_page

    header = get_json_header_token(token)
    return make_request_custom_success(
        endpoint,
        eventdevices_success_handler,
        method="GET",
        success_callback=success_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        redirect_callback=redirect_callback,
        params=params,
        header=header,
    )


def make_eventdevice_update(
    token,
    eventdevice_uri,
    device_id=None,
    name=None,
    comp_number=None,
    success_callback=None,
    failure_callback=None,
    progress_callback=None,
    redirect_callback=None,
):
    """
    Request that updates a PodiumEventDevice.

    Args:
        token (PodiumToken): The authentication token for this session.

        eventdevice_uri (str): URI for the eventdevice you are updating.

    Kwargs:
        device_id (int): Id of device to update
        name (str): Name of the device for this particular event, allows for
        car number/name to change between events. If blank/missing, will
        default to device name.

        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(result (dict), updated_uri (str))
        Defaults to None..

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(redirect_object (PodiumRedirect))
        Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """
    body = {}
    if device_id is not None:
        body["eventdevice[device_id]"] = device_id
    if name is not None:
        body["eventdevice[name]"] = name
    if comp_number is not None:
        body["eventdevice[comp_number]"] = comp_number
    header = get_json_header_token(token)
    return make_request_custom_success(
        eventdevice_uri,
        eventdevice_update_success_handler,
        method="PUT",
        success_callback=success_callback,
        redirect_callback=redirect_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        body=body,
        header=header,
        data={"updated_uri": eventdevice_uri},
    )


def make_eventdevice_get(
    token,
    eventdevice_uri,
    expand=True,
    quiet=None,
    success_callback=None,
    redirect_callback=None,
    failure_callback=None,
    progress_callback=None,
):

    """
    Request that returns a PodiumEventDevice for the provided eventdevice_uri

    Args:
        token (PodiumToken): The authentication token for this session.

        eventdevice_uri (str): URI for the eventdevice you want.

    Kwargs:

        expand (bool): Expand all objects in response output. Defaults to True

        quiet (object): If not None HTML layout will not render endpoint
        description. Defaults to None.

        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(PodiumEvent)
        Defaults to None.

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(result (dict), data (dict))
        Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """
    params = {}
    if expand is not None:
        params["expand"] = expand
    if quiet is not None:
        params["quiet"] = quiet
    header = get_json_header_token(token)
    return make_request_custom_success(
        eventdevice_uri,
        eventdevice_success_handler,
        method="GET",
        success_callback=success_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        redirect_callback=redirect_callback,
        params=params,
        header=header,
    )


def eventdevice_success_handler(req, results, data):
    """
    Creates and returns a PodiumDevice to the success_callback found in data
    if there is one.

    Called automatically by **make_device_get**

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.
    """
    if data["success_callback"] is not None:
        data["success_callback"](get_eventdevice_from_json(results["eventdevice"]))


def make_eventdevice_delete(
    token, eventdevice_uri, success_callback=None, failure_callback=None, progress_callback=None, redirect_callback=None
):

    """
    Deletes the device for the provided URI.

    Args:
        token (PodiumToken): The authentication token for this session.

        eventdevice_uri (str): URI for the eventdevice you want.

    Kwargs:
        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(deleted_uri (str))
        Defaults to None.

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(result (dict), data (dict))
        Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """
    header = get_json_header_token(token)
    return make_request_custom_success(
        eventdevice_uri,
        eventdevice_delete_handler,
        method="DELETE",
        success_callback=success_callback,
        redirect_callback=redirect_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        header=header,
        data={"deleted_uri": eventdevice_uri},
    )


def eventdevice_delete_handler(req, results, data):
    """
    Returns the URI for the deleted resource to the user set success_callback

    Called automatically by **make_eventdevice_delete**

    Args:
        req (UrlRequest): Instance of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.
    """
    if data["success_callback"] is not None:
        data["success_callback"](data["deleted_uri"])


def make_eventdevice_create(
    token,
    event_id,
    device_id,
    name,
    comp_number=None,
    success_callback=None,
    failure_callback=None,
    progress_callback=None,
    redirect_callback=None,
):
    """
    Request that creates a new PodiumEventDevice.

    The uri for the newly created event device will be provided to the
    redirect_callback if one is provided in the form of a PodiumRedirect.

    Args:
        token (PodiumToken): The authentication token for this session.

        event_id (int): Id of the event to add the device to.

        device_id (int): Id of the device to add to the event.

        name (str): Name of the device for this particular event, allows for
        car number/name to change between events. If blank/missing, will
        default to device name.

    Kwargs:
        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(result (dict), data (dict))
        Defaults to None..

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(redirect_object (PodiumRedirect))
        Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """
    endpoint = "{}/api/v1/events/{}/devices".format(podium_api.PODIUM_APP.podium_url, event_id)
    body = {
        "eventdevice[device_id]": device_id,
        "eventdevice[name]": name,
        "eventdevice[comp_number]": comp_number,
    }
    header = get_json_header_token(token)
    return make_request_custom_success(
        endpoint,
        None,
        method="POST",
        success_callback=success_callback,
        redirect_callback=create_eventdevice_redirect_handler,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        body=body,
        header=header,
        data={"_redirect_callback": redirect_callback},
    )


def create_eventdevice_redirect_handler(req, results, data):
    """
    Handles the success redirect of a **make_event_device_create** call.

    Returns a PodiumRedirect with a uri for the newly created event to the
    _redirect_callback found in data.

    Automatically called by **make_event_device_create**, will call the
    redirect_callback passed in to **make_event_device_create** if there is on.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["_redirect_callback"] is not None:
        data["_redirect_callback"](get_redirect_from_json(results, "eventdevice"))


def eventdevice_update_success_handler(req, results, data):
    """
    Success callback after updating an event. Will return the message from
    the server and the event uri to the success_callback.

    Called automatically by **make_eventdevice_update**.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["success_callback"] is not None:
        data["success_callback"](results, data["updated_uri"])


def eventdevices_success_handler(req, results, data):
    """
    Creates and returns a PodiumPagedResponse with PodiumEventDevice as the
    payload
    to the success_callback found in data if there is one.

    Called automatically by **make_eventdevices_get**.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["success_callback"] is not None:
        data["success_callback"](get_paged_response_from_json(results, "eventdevices"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
class PodiumFriendship(object):
    """
    Object that represents a Friendship

    **Attributes:**

        **friendship_id** (int): Integer Id of the friendship

        **user_id** (int): Integer id for user following the friend

        **user_uri** (str): URI to user following the friend.

        **friend_id** (int): Integer id for the user being followed.

        **friend_uri** (str): URI to user being followed.

    """

    def __init__(self, friendship_id, user_id, user_uri, friend_id, friend_uri):
        self.friendship_id = friendship_id
        self.user_id = user_id
        self.user_uri = user_uri
        self.friend_id = friend_id
        self.friend_uri = friend_uri


def get_friendship_from_json(json):
    """
    Returns a PodiumFriendship object from the json dict received from
    podium api.

    Args:
        json (dict): Dict of data from REST api

    Return:
        PodiumFriendship: The PodiumFriendship object for the data.

    """
    return PodiumFriendship(json["id"], json["user_id"], json["user_uri"], json["friend_id"], json["friend_uri"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import podium_api
from podium_api.asyncreq import get_json_header_token, make_request_custom_success
from podium_api.types.racestat import get_racestat_from_json
from podium_api.types.redirect import get_redirect_from_json


def make_racestat_get(
    token,
    endpoint,
    expand=False,
    quiet=None,
    success_callback=None,
    failure_callback=None,
    progress_callback=None,
    redirect_callback=None,
):
    """
     Request that returns a Racestat that represents a specific
    racestat found at the URI.

    Args:
        token (PodiumToken): The authentication token for this session.

    Kwargs:
        expand (bool): Expand all objects in response output. Defaults to False

        quiet (object): If not None HTML layout will not render endpoint
        description. Defaults to None.

        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(account (PodiumAccount))
        Defaults to None.

        failure_callback (function): Callback for redirects, failures, and
        errors. Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'redirect', 'failure'. Defaults
        to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(result (dict), data (dict))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """

    params = {"expand": expand}
    if quiet is not None:
        params["quiet"] = quiet
    header = get_json_header_token(token)
    return make_request_custom_success(
        endpoint,
        racestat_success_handler,
        method="GET",
        success_callback=success_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        redirect_callback=redirect_callback,
        params=params,
        header=header,
    )


def make_racestats_create(
    token,
    event_id,
    racestats,
    success_callback=None,
    failure_callback=None,
    progress_callback=None,
    redirect_callback=None,
):
    """
    add a collection of racestats to the specified event id
    Args:
        token (PodiumToken): The authentication token for this session

        event_id: The id of the event to apply racestats
    """
    endpoint = "{}/api/v1/events/{}/racestats".format(podium_api.PODIUM_APP.podium_url, event_id)

    index = 0
    body = {}
    for racestat in racestats:
        body[f"racestat[{index}][device_id]"] = racestat["device_id"]
        body[f"racestat[{index}][comp_number]"] = racestat["comp_number"]
        body[f"racestat[{index}][comp_class]"] = racestat["comp_class"]
        body[f"racestat[{index}][total_laps]"] = racestat["total_laps"]
        body[f"racestat[{index}][last_lap_time]"] = racestat["last_lap_time"]
        body[f"racestat[{index}][position_overall]"] = racestat["position_overall"]
        body[f"racestat[{index}][position_in_class]"] = racestat["position_in_class"]
        body[f"racestat[{index}][comp_number_ahead]"] = racestat["comp_number_ahead"]
        body[f"racestat[{index}][comp_number_behind]"] = racestat["comp_number_behind"]
        body[f"racestat[{index}][gap_to_ahead]"] = racestat["gap_to_ahead"]
        body[f"racestat[{index}][gap_to_behind]"] = racestat["gap_to_behind"]
        body[f"racestat[{index}][laps_to_ahead]"] = racestat["laps_to_ahead"]
        body[f"racestat[{index}][laps_to_behind]"] = racestat["laps_to_behind"]
        body[f"racestat[{index}][fc_flag]"] = racestat["fc_flag"]
        body[f"racestat[{index}][comp_flag]"] = racestat["comp_flag"]
        index += 1

    header = get_json_header_token(token)
    return make_request_custom_success(
        endpoint,
        None,
        method="POST",
        success_callback=success_callback,
        redirect_callback=create_racestat_redirect_handler,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        body=body,
        header=header,
        data={"_redirect_callback": redirect_callback},
    )


def make_racestat_create(
    token,
    event_id,
    device_id,
    comp_number,
    comp_class,
    total_laps,
    last_lap_time,
    position_overall,
    position_in_class,
    comp_number_ahead,
    comp_number_behind,
    gap_to_ahead,
    gap_to_behind,
    laps_to_ahead,
    laps_to_behind,
    fc_flag,
    comp_flag,
    success_callback=None,
    failure_callback=None,
    progress_callback=None,
    redirect_callback=None,
):
    """
    add a racestat for the specified event_id / device_id

    The uri for the newly created racestat will be provided to the
    redirect_callback if one is provided in the form of a PodiumRedirect.

    Args:
        token (PodiumToken): The authentication token for this session.

    Kwargs:
        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(result (dict), data (dict))
        Defaults to None..

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(redirect_object (PodiumRedirect))
        Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """

    endpoint = "{}/api/v1/events/{}/devices/{}/racestat".format(podium_api.PODIUM_APP.podium_url, event_id, device_id)
    body = {
        "racestat[comp_number]": comp_number,
        "racestat[comp_class]": comp_class,
        "racestat[total_laps]": total_laps,
        "racestat[last_lap_time]": last_lap_time,
        "racestat[position_overall]": position_overall,
        "racestat[position_in_class]": position_in_class,
        "racestat[comp_number_ahead]": comp_number_ahead,
        "racestat[comp_number_behind]": comp_number_behind,
        "racestat[gap_to_ahead]": gap_to_ahead,
        "racestat[gap_to_behind]": gap_to_behind,
        "racestat[laps_to_ahead]": laps_to_ahead,
        "racestat[laps_to_behind]": laps_to_behind,
        "racestat[fc_flag]": fc_flag,
        "racestat[comp_flag]": comp_flag,
    }
    header = get_json_header_token(token)
    return make_request_custom_success(
        endpoint,
        None,
        method="POST",
        success_callback=success_callback,
        redirect_callback=create_racestat_redirect_handler,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        body=body,
        header=header,
        data={"_redirect_callback": redirect_callback},
    )


def create_racestat_redirect_handler(req, results, data):
    """
    Handles the success redirect of a **make_racestat_create** call.

    Returns a PodiumRedirect with a uri for the newly created event to the
    _redirect_callback found in data.

    Automatically called by **make_racestat_create**, will call the
    redirect_callback passed in to **make_racestat_create** if there is one.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["_redirect_callback"] is not None:
        data["_redirect_callback"](get_redirect_from_json(results, "racestat"))


def racestat_success_handler(req, results, data):
    """
    Creates and returns a Racestat to the success_callback
    found in data.

    Called automatically by **make_account_request**.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    account = results["racestat"]
    if account is not None:
        if data["success_callback"] is not None:
            data["success_callback"](get_racestat_from_json(results["racestat"]))
    elif data["failure_callback"] is not None:
        data["failure_callback"]("None", results, data)



from typing import Callable

from kivy.logger import Logger

import podium_api
from podium_api.account import make_account_get
from podium_api.alertmessages import (
    make_alertmessage_create,
    make_alertmessage_get,
    make_alertmessages_get,
)
from podium_api.devices import (
    make_device_create,
    make_device_delete,
    make_device_get,
    make_device_update,
    make_devices_get,
)
from podium_api.eventdevices import (
    make_eventdevice_create,
    make_eventdevice_delete,
    make_eventdevice_get,
    make_eventdevice_update,
    make_eventdevices_get,
)
from podium_api.events import (
    make_event_create,
    make_event_delete,
    make_event_get,
    make_event_update,
    make_events_get,
)
from podium_api.friendships import (
    make_friendship_create,
    make_friendship_delete,
    make_friendship_get,
    make_friendships_get,
)
from podium_api.laps import make_lap_get, make_laps_get
from podium_api.presets import (
    make_preset_create,
    make_preset_delete,
    make_preset_get,
    make_preset_update,
    make_presets_get,
)
from podium_api.ratings import make_rating_create
from podium_api.types.account import PodiumAccount
from podium_api.types.user import PodiumUser
from podium_api.users import make_user_get
from podium_api.venues import make_venue_get, make_venues_get


class PodiumAPI(object):
    """
    The PodiumApi object holds references to the interfaces to the
    various asynchronous requests. You should provide a PodiumToken received
    from **podium_api.login.make_login_post** to create this object.

    Keep in mind all API requests are asynchronous, you need to provide
    callback functions that will receive the data once the request has
    completed. Most requests return their results in the on_success callback,
    but some creation requests return their success as a redirect to the
    newly created resource's URI. Reference the documentation for each
    function for more details.

    **Attributes:**
        **token** (PodiumToken): The token for the logged in user.

        **account** (PodiumAccountAPI): API object for account requests.

        **events** (PodiumEventsAPI): API object for event requests.

        **devices** (PodiumDevicesAPI): API object for device requests.

        **friendships** (PodiumFriendshipsAPI): API object for friendship
        requests.

        **users** (PodiumUsersApi): API object for user requests.

        **eventdevices** (PodiumEventDevicesAPI): API object for event-device
        requests.

        **laps** (PodiumLapsAPI): API object for lap requests.

        **alertmessages** (AlertMessagesAPI: API object for alertmessage requests.

    """

    def __init__(self, token):
        self.token = token
        self.account = PodiumAccountAPI(token)
        self.events = PodiumEventsAPI(token)
        self.devices = PodiumDevicesAPI(token)
        self.friendships = PodiumFriendshipsAPI(token)
        self.users = PodiumUsersAPI(token)
        self.eventdevices = PodiumEventDevicesAPI(token)
        self.laps = PodiumLapsAPI(token)
        self.alertmessages = PodiumAlertMessagesAPI(token)
        self.presets = PodiumPresetsAPI(token)
        self.ratings = PodiumRatingsAPI(token)

        self.podium_account = None
        self.podium_user = None

    def init_connection(
        self,
        success_callback: Callable[[PodiumAccount, PodiumUser], None],
        failure_callback: Callable[[str, str, str], None],
    ) -> None:
        self._load_account(success_callback, failure_callback)

    def _load_account(
        self, success_callback: Callable[[PodiumAccount], None], failure_callback: Callable[[str, str, str], None]
    ) -> None:
        def success(account: PodiumAccount):
            Logger.info("PodiumAPI: Loaded Account %s", account.username)
            self.podium_account = account
            self._load_user(account, success_callback, failure_callback)

        def failure(error_type: str, results: str, data: str) -> None:
            Logger.error("PodiumAPI: Failed to load account: %s: %s", error_type, results)
            failure_callback(error_type, results, data)

        self.account.get(success_callback=success, failure_callback=failure)

    def _load_user(
        self,
        account: PodiumAccount,
        success_callback: Callable[[PodiumAccount, PodiumUser], None],
        failure_callback: Callable[[str, str, str], None],
    ) -> None:
        def success(user):
            Logger.info("PodiumAPI: Loaded User %s", user.username)
            self.podium_user = user
            success_callback(account, user)

        def failure(error_type: str, results: str, data: str):
            Logger.error("PodiumAPI: Failed to load user: %s: %s", error_type, results)
            failure_callback(error_type, results, data)

        self.users.get(account.user_uri, success_callback=success, failure_callback=failure)


class PodiumLapsAPI(object):
    """
    Object that handles lap requests and keeps track of the
    authentication token necessary to do so. Usually accessed via
    PodiumAPI object.

    **Attributes:**
        **token** (PodiumToken): The token for the logged in user.

    """

    def __init__(self, token):
        self.token = token

    def list(self, *args, **kwargs):
        """
        Request that returns a PodiumPagedRequest of laps.

        Args:
            endpoint (str): The endpoint to make the request too.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumPagedResponse)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

            start (int): Starting index for events list. 0 indexed.

            per_page (int): Number per page of results, max of 100.

        Return:
            UrlRequest: The request being made.

        """
        make_laps_get(self.token, *args, **kwargs)

    def get(self, *args, **kwargs):
        """
        Request that returns a PodiumLap that represents a specific
        lap found at the URI.

        Args:
            endpoint (str): The URI for the lap.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumLap)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_lap_get(self.token, *args, **kwargs)


class PodiumEventDevicesAPI(object):
    """
    Object that handles event-device requests and keeps track of the
    authentication token necessary to do so. Usually accessed via
    PodiumAPI object.

    **Attributes:**
        **token** (PodiumToken): The token for the logged in user.

    """

    def __init__(self, token):
        self.token = token

    def list(self, *args, **kwargs):
        """
        Request that returns a PodiumPagedRequest of events.
        By default a get request to
        'https://podium.live/api/v1/events/{event_id}/devices' will be made.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumPagedResponse)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict),
                           data (dict))
            Values for failure type are: 'error', 'failure'.
            Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int),
                            data (dict))
            Defaults to None.

            start (int): Starting index for events list. 0 indexed.

            per_page (int): Number per page of results, max of 100.

            endpoint (str): If provided this endpoint will be used instead
            of the default:
            'https://podium.live/api/v1/presets'

            event_id (int): If an endpoint is not provided you should
            provide the id of the event for which you want to look up
            the devices.

        Return:
            UrlRequest: The request being made.

        """
        make_eventdevices_get(self.token, *args, **kwargs)

    def create(self, *args, **kwargs):
        """
        Request that creates a new PodiumEventDevice.

        The uri for the newly created event device will be provided to the
        redirect_callback if one is provided in the form of a PodiumRedirect.

        Args:
            event_id (int): Id of the event to add the device to.

            device_id (int): Id of the device to add to the event.

            name (str): Name of the device for this particular event, allows for
            car number/name to change between events. If blank/missing, will
            default to device name.

        Kwargs:
            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(result (dict), data (dict))
            Defaults to None..

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(redirect_object (PodiumRedirect))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_eventdevice_create(self.token, *args, **kwargs)

    def update(self, *args, **kwargs):
        """
        Request that updates a PodiumEventDevice.

        Args:
            eventdevice_uri (str): URI for the eventdevice you are updating.

        Kwargs:
            name (str): Name of the device for this particular event, allows for
            car number/name to change between events. If blank/missing, will
            default to device name.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(result (dict), updated_uri (str))
            Defaults to None..

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(redirect_object (PodiumRedirect))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_eventdevice_update(self.token, *args, **kwargs)

    def get(self, *args, **kwargs):
        """
        Request that returns a PodiumEventDevice for the provided
        eventdevice_uri

        Args:
            eventdevice_uri (str): URI for the eventdevice you want.

        Kwargs:

            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumEvent)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_eventdevice_get(self.token, *args, **kwargs)

    def delete(self, *args, **kwargs):
        """
        Deletes the device for the provided URI.

        Args:
            eventdevice_uri (str): URI for the eventdevice you want.

        Kwargs:
            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(deleted_uri (str))
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_eventdevice_delete(self.token, *args, **kwargs)


class PodiumUsersAPI(object):
    """
    Object that handles user requests and keeps track of the
    authentication token necessary to do so. Usually accessed via
    PodiumAPI object.

    **Attributes:**
        **token** (PodiumToken): The token for the logged in user.

    """

    def __init__(self, token):
        self.token = token

    def get(self, *args, **kwargs):
        """
        Returns a PodiumUser object found at the uri provided in the endpoint
        arg.

        Args:
            endpoint (str): The URI to make the request to. Typically should be
            provided by some api object.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to False

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(token (string))
            Defaults to None.

            failure_callback (function): Callback for redirects, failures, and
            errors. Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'redirect', 'failure'.
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_user_get(self.token, *args, **kwargs)


class PodiumFriendshipsAPI(object):
    """
    Object that handles friendship requests and keeps track of the
    authentication token necessary to do so. Usually accessed via
    PodiumAPI object.

    **Attributes:**
        **token** (PodiumToken): The token for the logged in user.

    """

    def __init__(self, token):
        self.token = token

    def get(self, *args, **kwargs):
        """
        Request that returns a PodiumFriendship that represents a specific
        friendship found at the URI.

        Args:
            endpoint (str): The URI for the friendship.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumPagedResponse)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_friendship_get(self.token, *args, **kwargs)

    def list(self, *args, **kwargs):
        """
        Request that returns a PodiumPagedRequest of friendships.

        Args:
            endpoint (str): The endpoint to make the request too.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumPagedResponse)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

            start (int): Starting index for events list. 0 indexed.

            per_page (int): Number per page of results, max of 100.

        Return:
            UrlRequest: The request being made.

        """
        make_friendships_get(self.token, *args, **kwargs)

    def create(self, *args, **kwargs):
        """
        Request that adds a friendship for the user whose token is in use.

        The uri for the newly created event will be provided to the
        redirect_callback if one is provided in the form of a PodiumRedirect.

        Kwargs:
            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(result (dict), data (dict))
            Defaults to None..

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(redirect_object (PodiumRedirect))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_friendship_create(self.token, *args, **kwargs)

    def delete(self, *args, **kwargs):
        """
        Deletes the friendship for the provided URI.

        Args:
            friendship_uri (str): URI for the friendship you want to delete.

        Kwargs:
            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(deleted_uri (str))
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_friendship_delete(self.token, *args, **kwargs)


class PodiumAccountAPI(object):
    """
    Object that handles account requests and keeps track of the
    authentication token necessary to do so. Usually accessed via
    PodiumAPI object.

    **Attributes:**
        **token** (PodiumToken): The token for the logged in user.

    """

    def __init__(self, token):
        self.token = token

    def get(self, *args, **kwargs):
        """
        Request that returns the account for the provided authentication token.
        Hits the api/v1/account endpoint with a GET request.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to False

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(account (PodiumAccount))
            Defaults to None.

            failure_callback (function): Callback for redirects, failures, and
            errors. Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'redirect', 'failure'.
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_account_get(self.token, *args, **kwargs)


class PodiumDevicesAPI(object):
    """
    Object that handles device requests and keeps track of the
    authentication token necessary to do so. Usually accessed via
    PodiumAPI object.

    **Attributes:**
        **token** (PodiumToken): The token for the logged in user.

    """

    def __init__(self, token):
        self.token = token

    def create(self, *args, **kwargs):
        """
        Request that creates a new PodiumDevice.

        The uri for the newly created event will be provided to the
        redirect_callback if one is provided in the form of a PodiumRedirect.

        Args:
            name(str): Name of the device.

        Kwargs:
            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(result (dict), data (dict))
            Defaults to None..

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(redirect_object (PodiumRedirect))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_device_create(self.token, *args, **kwargs)

    def update(self, *args, **kwargs):
        """
        Request that updates a PodiumDevice

        Args:
            device_uri (str): URI for the device you are updating.

        Kwargs:
            name(str): Name of the device.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(result (dict), updated_uri (str))
            Defaults to None..

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(redirect_object (PodiumRedirect))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_device_update(self.token, *args, **kwargs)

    def get(self, *args, **kwargs):
        """
        Request that returns a PodiumDevice for the provided device_uri

        Args:
            device_uri (str): URI for the device you want.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumEvent)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_device_get(self.token, *args, **kwargs)

    def delete(self, *args, **kwargs):
        """
        Deletes the device for the provided URI.

        Args:
            device_uri (str): URI for the device you want.

        Kwargs:
            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(deleted_uri (str))
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_device_delete(self.token, *args, **kwargs)

    def list(self, *args, **kwargs):
        """
        Request that returns a PodiumPagedRequest of PodiumDevice.

        Args:
            endpoint (str): the endpoint to make the request to.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumPagedResponse)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict),
                           data (dict))
            Values for failure type are: 'error', 'failure'.
            Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int),
                            data (dict))
            Defaults to None.

            start (int): Starting index for events list. 0 indexed.

            per_page (int): Number per page of results, max of 100.

        Return:
            UrlRequest: The request being made.

        """
        make_devices_get(self.token, *args, **kwargs)


class PodiumEventsAPI(object):
    """
    Object that handles event requests and keeps track of the
    authentication token necessary to do so. Usually accessed via
    PodiumAPI object.

    **Attributes:**
        **token** (PodiumToken): The token for the logged in user.

    """

    def __init__(self, token):
        self.token = token

    def list(self, *args, **kwargs):
        """
        Request that returns a PodiumPagedRequest of events.
        By default a get request to
        'https://podium.live/api/v1/events' will be made.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumPagedResponse)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

            start (int): Starting index for events list. 0 indexed.

            per_page (int): Number per page of results, max of 100.

            endpoint (str): If provided the start, per_page, expand, and quiet
            params will not be used instead making a request based on the
            provided endpoint.

        Return:
            UrlRequest: The request being made.

        """
        make_events_get(self.token, *args, **kwargs)

    def get(self, *args, **kwargs):
        """
        Request that returns a PodiumEvent for the provided event_uri.

        Args:
            event_uri (str): URI for the event you want.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumEvent)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_event_get(self.token, *args, **kwargs)

    def delete(self, *args, **kwargs):
        """
        Deletes the event for the provided URI.

        Args:
            event_uri (str): URI for the event you want.

        Kwargs:
            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(deleted_uri (str))
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_event_delete(self.token, *args, **kwargs)

    def create(self, *args, **kwargs):
        """
        Request that creates a new PodiumEvent.

        The uri for the newly created event will be provided to the
        redirect_callback if one is provided in the form of a PodiumRedirect.

        Args:
            title (str): title for the vent.

            start_time (str): Starting time, use ISO 8601 format.

            end_time (str): Ending time, use ISO 8601 format.

        Kwargs:
            venue_id(str): ID for the venue of event.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(result (dict), data (dict))
            Defaults to None..

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(redirect_object (PodiumRedirect))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_event_create(self.token, *args, **kwargs)

    def update(self, *args, **kwargs):
        """
        Request that updates a PodiumEvent.

        The uri for the newly created event will be provided to the
        redirect_callback if one is provided in the form of a PodiumRedirect.

        Args:
            event_uri (str): URI for the event you are updating.

        Kwargs:
            venue_id(str): ID for the venue of event.

            title (str): title for the vent.

            start_time (str): Starting time, use ISO 8601 format.

            end_time (str): Ending time, use ISO 8601 format.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(result (dict), updated_uri (str))
            Defaults to None..

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(redirect_object (PodiumRedirect))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_event_update(self.token, *args, **kwargs)


class PodiumAlertMessagesAPI(object):
    """
    Object that handles event requests and keeps track of the
    authentication token necessary to do so. Usually accessed via
    PodiumAPI object.

    **Attributes:**
        **token** (PodiumToken): The token for the logged in user.

    """

    def __init__(self, token):
        self.token = token

    def list(self, *args, **kwargs):
        """
        Request that returns a PodiumPagedRequest of events.
        By default a get request to
        'https://podium.live/api/v1/events' will be made.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumPagedResponse)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

            start (int): Starting index for events list. 0 indexed.

            per_page (int): Number per page of results, max of 100.

            endpoint (str): If provided the start, per_page, expand, and quiet
            params will not be used instead making a request based on the
            provided endpoint.

        Return:
            UrlRequest: The request being made.

        """
        make_alertmessages_get(self.token, *args, **kwargs)

    def get(self, *args, **kwargs):
        """
        Request that returns an AlertMessage for the provided alertmessage_uri.

        Args:
            endpoint (str): URI for the alertmessage you want.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumEvent)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_alertmessage_get(self.token, *args, **kwargs)

    def create(self, *args, **kwargs):
        """
        Request that creates a new PodiumAlertMessage.

        The uri for the newly created alertmessage will be provided to the
        redirect_callback if one is provided in the form of a PodiumRedirect.

        Args:
            message (str): message for the alertmessage

            priority (int): priority of the alertmessage

        Kwargs:
            message(str): ID for the venue of event.
            priority(int): priority for the message.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(result (dict), data (dict))
            Defaults to None..

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(redirect_object (PodiumRedirect))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_alertmessage_create(self.token, *args, **kwargs)


class PodiumVenuesAPI(object):
    """
    Object that handles event requests and keeps track of the
    authentication token necessary to do so. Usually accessed via
    PodiumAPI object.

    **Attributes:**
        **token** (PodiumToken): The token for the logged in user.

    """

    def __init__(self, token):
        self.token = token

    def list(self, *args, **kwargs):
        """
        Request that returns a PodiumPagedRequest of events.
        By default a get request to
        'https://podium.live/api/v1/events' will be made.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumPagedResponse)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

            start (int): Starting index for events list. 0 indexed.

            per_page (int): Number per page of results, max of 100.

            endpoint (str): If provided the start, per_page, expand, and quiet
            params will not be used instead making a request based on the
            provided endpoint.

        Return:
            UrlRequest: The request being made.

        """
        make_venues_get(self.token, *args, **kwargs)

    def get(self, *args, **kwargs):
        """
        Request that returns an Venue for the provided endpoint.

        Args:
            endpoint (str): URI for the venue you want.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumEvent)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_venue_get(self.token, *args, **kwargs)


class PodiumPresetsAPI(object):
    """
    Object that handles preset requests and keeps track of the
    authentication token necessary to do so. Usually accessed via
    PodiumAPI object.

    **Attributes:**
        **token** (PodiumToken): The token for the logged in user.

    """

    def __init__(self, token):
        self.token = token

    def list(self, *args, **kwargs):
        """
        Request that returns a PodiumPagedRequest of presets.
        By default a get request to
        'https://podium.live/api/v1/presets' will be made.

        Args:
            type (str): Type key for the presets you want.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumPagedResponse)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

            start (int): Starting index for presets list. 0 indexed.

            per_page (int): Number per page of results, max of 100.

            endpoint (str): If provided the start, per_page, expand, and quiet
            params will not be used instead making a request based on the
            provided endpoint.

        Return:
            UrlRequest: The request being made.

        """
        make_presets_get(self.token, *args, **kwargs)

    def list_my(self, *args, **kwargs):
        """
        Request that returns a PodiumPagedRequest of presets for the logged in user.
        By default a get request to
        'https://podium.live/api/v1/user/my/presets' will be made.

        Args:
            type (str): Type key for the presets you want.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumPagedResponse)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

            start (int): Starting index for presets list. 0 indexed.

            per_page (int): Number per page of results, max of 100.

            endpoint (str): If provided the start, per_page, expand, and quiet
            params will not be used instead making a request based on the
            provided endpoint.

        Return:
            UrlRequest: The request being made.
        """
        endpoint = "{}/api/v1/users/me/presets".format(podium_api.PODIUM_APP.podium_url)
        make_presets_get(self.token, endpoint=endpoint, *args, **kwargs)

    def get(self, *args, **kwargs):
        """
        Request that returns a PodiumPreset for the provided preset_uri.

        Args:
            preset_uri (str): URI for the preset you want.

        Kwargs:
            expand (bool): Expand all objects in response output.
            Defaults to True

            quiet (object): If not None HTML layout will not render endpoint
            description. Defaults to None.

            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(PodiumPreset)
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_preset_get(self.token, *args, **kwargs)

    def delete(self, *args, **kwargs):
        """
        Deletes the preset for the provided URI.

        Args:
            preset_uri (str): URI for the preset you want.

        Kwargs:
            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(deleted_uri (str))
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(result (dict), data (dict))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_preset_delete(self.token, *args, **kwargs)

    def create(self, *args, **kwargs):
        """
        Request that creates a new PodiumPreset.

        The uri for the newly created preset will be provided to the
        redirect_callback if one is provided in the form of a PodiumRedirect.

        Args:
            name (str): Name for the preset

            notes (str): Notes for the preset

            preset_data (str): JSON data of the preset

            mapping_type_id (int): The Id representing the mapping type

            private(int): 1 if the preset is private to the creating user

        Kwargs:
            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(result (dict), data (dict))
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(redirect_object (PodiumRedirect))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_preset_create(self.token, *args, **kwargs)

    def update(self, *args, **kwargs):
        """
        Request that updates a PodiumPreset.

        The uri for the newly created preset will be provided to the
        redirect_callback if one is provided in the form of a PodiumRedirect.

        Args:
            preset_uri (str): URI for the preset you are updating.

            name (str): Name for the preset

            notes (str): Notes for the preset

            preset_data (str): JSON data of the preset

            mapping_type_id (int): The Id representing the mapping type

            private(int): 1 if the preset is private to the creating user

        Kwargs:
            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(result (dict), updated_uri (str))
            Defaults to None..

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(redirect_object (PodiumRedirect))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_preset_update(self.token, *args, **kwargs)


class PodiumRatingsAPI(object):
    """
    Object that handles ratings requests and keeps track of the
    authentication token necessary to do so. Usually accessed via
    PodiumAPI object.

    **Attributes:**
        **token** (PodiumToken): The token for the logged in user.

    """

    def __init__(self, token):
        self.token = token

    def create(self, *args, **kwargs):
        """
        Request that creates or updates a PodiumRating.

        The uri for the newly created rating will be provided to the
        redirect_callback if one is provided in the form of a PodiumRedirect.

        Args:
            rateable_type (string): type of rateable object. e.g. preset

            rateable_id (int): ID of the object to rate

            rating (float): Rating value

        Kwargs:
            success_callback (function): Callback for a successful request,
            will have the signature:
                on_success(result (dict), data (dict))
            Defaults to None.

            failure_callback (function): Callback for failures and errors.
            Will have the signature:
                on_failure(failure_type (string), result (dict), data (dict))
            Values for failure type are: 'error', 'failure'. Defaults to None.

            redirect_callback (function): Callback for redirect,
            Will have the signature:
                on_redirect(redirect_object (PodiumRedirect))
            Defaults to None.

            progress_callback (function): Callback for progress updates,
            will have the signature:
                on_progress(current_size (int), total_size (int), data (dict))
            Defaults to None.

        Return:
            UrlRequest: The request being made.

        """
        make_rating_create(self.token, *args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class PodiumApplicationNotRegistered(Exception):
    """This exception is raised if a request is attempted without first
    calling register_podium_application"""

    pass


class NoEndpointOrIdsProvided(Exception):
    """This exception is raised if a request is attempted
    without providing an endpoint or required ids.
    """

    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class Racestat(object):
    """
    Object that represents a race statistic snapshot

    **Attributes:**
        **racestat_id** (str): Unique id for this racestat

        **comp_number** (str): Competitor number assigned for the race

        **comp_class** (str): Competitor class assigned for the race

        **total_laps** (int): Total number of laps according to timing and scoring

        **last_lap_time** (float): Last lap time according to timing and scoring

        **position_overall** (int): Overall race position

        **position_in_class** (int): Position in class

        **comp_number_ahead** (str): competitor number of competitor ahead

        **comp_number_behind** (str): competitor number of competitor behind

        **gap_to_ahead** (float): time gap to competitor ahead

        **gap_to_behind** (float): time gap to competitor behind

        **laps_to_ahead** (int): laps to competitor ahead

        **laps_to_behind** (int): laps to competitor behind

        **fc_flag** (int): full course flag status

        **comp_flag** (int): competitor flag status

        **eventdevice_uri:** (str): URI of the eventdevice this racestat belongs to

        **device_uri:** (str): URI of the device this racestat belongs to

        **user_uri** (str): URI of the user this racestat belongs to
    """

    def __init__(
        self,
        racestat_id,
        uri,
        comp_number,
        comp_class,
        total_laps,
        last_lap_time,
        position_overall,
        position_in_class,
        comp_number_ahead,
        comp_number_behind,
        gap_to_ahead,
        gap_to_behind,
        laps_to_ahead,
        laps_to_behind,
        fc_flag,
        comp_flag,
        eventdevice_uri,
        device_uri,
        user_uri,
    ):
        self.racestat_id = racestat_id
        self.uri = uri
        self.comp_number = comp_number
        self.comp_class = comp_class
        self.total_laps = total_laps
        self.last_lap_time = last_lap_time
        self.position_overall = position_overall
        self.position_in_class = position_in_class
        self.comp_number_ahead = comp_number_ahead
        self.comp_number_behind = comp_number_behind
        self.gap_to_ahead = gap_to_ahead
        self.gap_to_behind = gap_to_behind
        self.laps_to_ahead = laps_to_ahead
        self.laps_to_behind = laps_to_behind
        self.fc_flag = fc_flag
        self.comp_flag = comp_flag
        self.eventdevice_uri = eventdevice_uri
        self.device_uri = device_uri
        self.user_uri = user_uri


def get_racestat_from_json(json):
    """
    Returns a Racestat object from the json dict received from podium api.

    Args:
        json (dict): Dict of data from REST api

    Return:
        Racestat: The Racestat object for the data.
    """
    return Racestat(
        json["id"],
        json["URI"],
        json["comp_number"],
        json["comp_class"],
        json["total_laps"],
        json["last_lap_time"],
        json["position_overall"],
        json["position_in_class"],
        json["comp_number_ahead"],
        json["comp_number_behind"],
        json["gap_to_ahead"],
        json["gap_to_behind"],
        json["laps_to_ahead"],
        json["laps_to_behind"],
        json["fc_flag"],
        json["comp_flag"],
        json["eventdevice_uri"],
        json["device_uri"],
        json["user_uri"],
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
class PodiumRating(object):
    """
    Object that represents a Preset.

    **Attributes:**
        **preset_id** (int): Unique id for preset

        **uri** (str): Endpoint for accessing full preset information.

        **name** (str): Name of the preset.

        **notes** (str): Notes for this preset

        **preset** (str): JSON data for preset

        **mapping_type_id** (int): ID of the mapping type

        **mapping_type** (str): string key representation of the mapping type

        **updated** (str): Date preset was updated. ISO 8601 format.

        **created** (str): Date preset was created. ISO 8601 format.
    """

    def __init__(self, user_id, rating):
        self.rating = rating


def get_rating_from_json(json):
    """
    Returns a PodiumEvent object from the json dict received from podium api.

    Args:
        json (dict): Dict of data from REST api

    Return:
        PodiumEvent: The PodiumEvent object for this data.
    """
    return PodiumRating(json["rating"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import podium_api
from podium_api.asyncreq import get_json_header_token, make_request_custom_success
from podium_api.types.paged_response import get_paged_response_from_json
from podium_api.types.preset import get_preset_from_json
from podium_api.types.redirect import get_redirect_from_json


def make_preset_update(
    token,
    preset_uri,
    name=None,
    notes=None,
    preset_data=None,
    private=None,
    preview_image_name=None,
    preview_image_data=None,
    success_callback=None,
    failure_callback=None,
    progress_callback=None,
    redirect_callback=None,
):
    """
    Request that updates a PodiumPreset.

    Args:
        token (PodiumToken): The authentication token for this session.

        preset_uri (str): URI for the preset you are updating.



    Kwargs:
        name (str): Name for the preset

        notes (str): Notes for the preset

        preset_data (str): JSON data of the preset

        private(int): 1 if the preset is private to the creating user

        preview_image_name(str): Name of the preview image file

        preview_image_data(str): Base 64 encoded preview image data

        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(result (dict), updated_uri (str))
        Defaults to None..

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(redirect_object (PodiumRedirect))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """
    body = {}
    if name is not None:
        body["preset[name]"] = name
    if notes is not None:
        body["preset[notes]"] = notes
    if preset_data is not None:
        body["preset[preset_data]"] = preset_data
    if private is not None:
        body["preset[private]"] = int(private)
    if preview_image_name is not None:
        body["preset[preview_image_name]"] = preview_image_name
    if preview_image_data is not None:
        body["preset[preview_image_data]"] = preview_image_data
    header = get_json_header_token(token)
    return make_request_custom_success(
        preset_uri,
        preset_update_success_handler,
        method="PUT",
        success_callback=success_callback,
        redirect_callback=redirect_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        body=body,
        header=header,
        data={"updated_uri": preset_uri},
    )


def make_preset_create(
    token,
    name,
    notes,
    preset,
    type,
    private,
    preview_image_name,
    preview_image_data,
    success_callback=None,
    failure_callback=None,
    progress_callback=None,
    redirect_callback=None,
):
    """
    Request that creates a new PodiumPreset.

    The uri for the newly created preset will be provided to the
    redirect_callback if one is provided in the form of a PodiumRedirect.

    Args:
        token (PodiumToken): The authentication token for this session.

        name (str): Name for the preset

        notes (str): Notes for the preset

        preset_data (str): JSON data of the preset

        type (str): Key of the mapping type

        private(int): 1 if the preset is private to the creating user

        preview_image_name(str): Name of the preview image file

        preview_image_data(str): Base 64 encoded preview image data

    Kwargs:
        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(result (dict), data (dict))
        Defaults to None..

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(redirect_object (PodiumRedirect))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """
    endpoint = "{}/api/v1/presets".format(podium_api.PODIUM_APP.podium_url)
    body = {
        "preset[name]": name,
        "preset[notes]": notes,
        "preset[preset_data]": preset,
        "preset[type]": type,
        "preset[private]": int(private),
        "preset[preview_image_name]": preview_image_name,
        "preset[preview_image_data]": preview_image_data,
    }
    header = get_json_header_token(token)
    return make_request_custom_success(
        endpoint,
        None,
        method="POST",
        success_callback=success_callback,
        redirect_callback=create_preset_redirect_handler,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        body=body,
        header=header,
        data={"_redirect_callback": redirect_callback},
    )


def make_preset_delete(
    token, preset_uri, success_callback=None, redirect_callback=None, failure_callback=None, progress_callback=None
):
    """
    Deletes the preset for the provided URI.

    Args:
        token (PodiumToken): The authentication token for this session.

        preset_uri (str): URI for the preset you want to delete.

    Kwargs:
        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(deleted_uri (str))
        Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(result (dict), data (dict))
        Defaults to None.

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """
    header = get_json_header_token(token)
    return make_request_custom_success(
        preset_uri,
        preset_delete_handler,
        method="DELETE",
        success_callback=success_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        redirect_callback=redirect_callback,
        header=header,
        data={"deleted_uri": preset_uri},
    )


def make_preset_get(
    token,
    preset_uri,
    expand=True,
    quiet=None,
    success_callback=None,
    redirect_callback=None,
    failure_callback=None,
    progress_callback=None,
):
    """
    Request that returns a PodiumPreset for the provided preset_uri.

    Args:
        token (PodiumToken): The authentication token for this session.

        preset_uri (str): URI for the preset you want.

    Kwargs:
        expand (bool): Expand all objects in response output. Defaults to True

        quiet (object): If not None HTML layout will not render endpoint
        description. Defaults to None.

        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(PodiumPreset)
        Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(result (dict), data (dict))
        Defaults to None.

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """
    params = {}
    if expand is not None:
        params["expand"] = expand
    if quiet is not None:
        params["quiet"] = quiet
    header = get_json_header_token(token)
    return make_request_custom_success(
        preset_uri,
        preset_success_handler,
        method="GET",
        success_callback=success_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        redirect_callback=redirect_callback,
        params=params,
        header=header,
    )


def make_presets_get(
    token,
    type,
    search=None,
    start=None,
    per_page=None,
    endpoint=None,
    expand=True,
    quiet=None,
    success_callback=None,
    redirect_callback=None,
    failure_callback=None,
    progress_callback=None,
):
    """
    Request that returns a PodiumPagedRequest of presets.
    By default a get request to
    'https://podium.live/api/v1/presets' will be made.

    Args:
        token (PodiumToken): The authentication token for this session.

        type (str): The type key to query.

    Kwargs:
        search (string): search string on name or notes

        expand (bool): Expand all objects in response output. Defaults to True

        quiet (object): If not None HTML layout will not render endpoint
        description. Defaults to None.

        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(PodiumPagedResponse)
        Defaults to None.

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(result (dict), data (dict))
        Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

        start (int): Starting index for preset list. 0 indexed.

        per_page (int): Number per page of results, max of 100.

        endpoint (str): If provided this endpoint will be used instead of
        the default: 'https://podium.live/api/v1/presets'

    Return:
        UrlRequest: The request being made.

    """
    if endpoint is None:
        endpoint = "{}/api/v1/presets".format(podium_api.PODIUM_APP.podium_url)
    params = {"type": type}
    if search is not None:
        params["search"] = search
    if expand is not None:
        params["expand"] = expand
    if quiet is not None:
        params["quiet"] = quiet
    if start is not None:
        params["start"] = start
    if per_page is not None:
        per_page = min(per_page, 100)
        params["per_page"] = per_page

    header = get_json_header_token(token)
    return make_request_custom_success(
        endpoint,
        presets_success_handler,
        method="GET",
        success_callback=success_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        redirect_callback=redirect_callback,
        params=params,
        header=header,
    )


def preset_delete_handler(req, results, data):
    """
    Returns the URI for the deleted resource to the user set success_callback

    Called automatically by **make_preset_delete**

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.
    """
    if data["success_callback"] is not None:
        data["success_callback"](data["deleted_uri"])


def preset_success_handler(req, results, data):
    """
    Creates and returns a PodiumPreset to the success_callback found in data
    if there is one.

    Called automatically by **make_preset_get**

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.
    """
    if data["success_callback"] is not None:
        data["success_callback"](get_preset_from_json(results["preset"]))


def presets_success_handler(req, results, data):
    """
    Creates and returns a PodiumPagedResponse with PodiumPreset as the payload
    to the success_callback found in data if there is one.

    Called automatically by **make_presets_get**.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["success_callback"] is not None:
        data["success_callback"](get_paged_response_from_json(results, "presets"))


def create_preset_redirect_handler(req, results, data):
    """
    Handles the success redirect of a **make_preset_create** call.

    Returns a PodiumRedirect with a uri for the newly created preset to the
    _redirect_callback found in data.

    Automatically called by **make_preset_create**, will call the
    redirect_callback passed in to **make_preset_create** if there is on.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["_redirect_callback"] is not None:
        data["_redirect_callback"](get_redirect_from_json(results, "preset"))


def preset_update_success_handler(req, results, data):
    """
    Success callback after updating a preset. Will return the message from
    the server and the preset uri to the success_callback.

    Called automatically by **make_preset_update**.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["success_callback"] is not None:
        data["success_callback"](results, data["updated_uri"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import podium_api
from podium_api.asyncreq import get_json_header_token, make_request_custom_success
from podium_api.types.account import get_account_from_json


def make_account_get(
    token,
    expand=False,
    quiet=None,
    success_callback=None,
    failure_callback=None,
    progress_callback=None,
    redirect_callback=None,
):
    """
    Request that returns the account for the provided authentication token.
    Hits the api/v1/account endpoint with a GET request.

    Args:
        token (PodiumToken): The authentication token for this session.

    Kwargs:
        expand (bool): Expand all objects in response output. Defaults to False

        quiet (object): If not None HTML layout will not render endpoint
        description. Defaults to None.

        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(account (PodiumAccount))
        Defaults to None.

        failure_callback (function): Callback for redirects, failures, and
        errors. Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'redirect', 'failure'. Defaults
        to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """
    endpoint = "{}/api/v1/account".format(podium_api.PODIUM_APP.podium_url)
    params = {"expand": expand}
    if quiet is not None:
        params["quiet"] = quiet
    header = get_json_header_token(token)
    return make_request_custom_success(
        endpoint,
        account_success_handler,
        method="GET",
        success_callback=success_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        redirect_callback=redirect_callback,
        params=params,
        header=header,
    )


def account_success_handler(req, results, data):
    """
    Creates and returns a PodiumAccount to the success_callback
    found in data.

    Called automatically by **make_account_request**.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    account = results["account"]
    if account is not None:
        if data["success_callback"] is not None:
            data["success_callback"](get_account_from_json(results["account"]))
    elif data["failure_callback"] is not None:
        data["failure_callback"]("None", results, data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from kivy.network.urlrequest import UrlRequest

import podium_api

try:
    from urllib.parse import urlencode
except:
    from urllib import urlencode

from podium_api.types.exceptions import PodiumApplicationNotRegistered


def get_json_header_token(token):
    """
    Returns a header prepared with the app_id and app_secret set to tell
    the server to return json. Content-Type will be
    'application/x-www-form-urlencoded'

    Return:
        dict: Dict containing the header data for a request.

    """
    if podium_api.PODIUM_APP is None:
        raise PodiumApplicationNotRegistered()
    return {
        "Content-Type": "application/x-www-form-urlencoded",
        "Authorization": "Bearer {}".format(token.token),
        "Accept": "application/json",
    }


def get_json_header():
    """
    Returns a header prepared with the app_id and app_secret set to tell
    the server to return json. Content-Type will be
    'application/x-www-form-urlencoded'

    Return:
        dict: Dict containing the header data for a request.

    """
    if podium_api.PODIUM_APP is None:
        raise PodiumApplicationNotRegistered()
    return {
        "Content-Type": "application/x-www-form-urlencoded",
        "Authorization": "Basic {}:{}".format(podium_api.PODIUM_APP.app_id, podium_api.PODIUM_APP.app_secret),
        "Accept": "application/json",
    }


def make_request(
    endpoint,
    method="GET",
    on_success=None,
    on_failure=None,
    on_error=None,
    on_redirect=None,
    on_progress=None,
    body=None,
    header=None,
    data=None,
    params=None,
):
    """
    Creates and starts a UrlRequest.

    Args:
        endpoint (str): The endpoint the request will go to.

    Kwargs:
        method (str): The type of request being made. Defaults to 'GET'

        on_success (function): Callback for a successful request, will have
        the signature:
            on_success(request (UrlRequest), result (dict), data (dict))
        Defaults to None.

        on_failure (function): Callback for request that fails, will have
        the signature:
            on_failure(request (UrlRequest), result (dict), data (dict))
        Defaults to None.

        on_error (function): Callback for request that handles errors, will have
        the signature:
            on_error(request (UrlRequest), result (dict), data (dict))
        Defaults to None.

        on_redirect (function): Callback for request that redirects, will have
        the signature:
            on_redirect(request (UrlRequest), result (dict), data (dict))
        Defaults to None.

        on_progress (function): Callback for request in progress, will have
        the signature:
            on_progress(request (UrlRequest), current (int), total (int),
                        data (dict))
        Defaults to None.

        body (dict): Body of the request, will be encoded using
        urllib.urlencode. Defaults to None.

        header (dict): The header for the request. Defaults to None.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Each callback will receive the
        data in here. Defaults to empty dict.

    Return:
        UrlRequest: The request being made.

    """
    if body is not None:
        body = urlencode(body)
    if params is not None and params != {}:
        params = urlencode(params)
        if "?" in endpoint:
            endpoint = "{}&{}".format(endpoint, params)
        else:
            endpoint = "{}?{}".format(endpoint, params)
    return UrlRequest(
        endpoint,
        method=method,
        req_body=body,
        req_headers=header,
        on_success=(lambda req, res: on_success(req, res, data)) if on_success is not None else None,
        on_failure=(lambda req, res: on_failure(req, res, data)) if on_failure is not None else None,
        on_redirect=(lambda req, res: on_redirect(req, res, data)) if on_redirect is not None else None,
        on_progress=(lambda req, cur, tot: on_progress(req, cur, tot, data)) if on_progress is not None else None,
        on_error=(lambda req, res: on_error(req, res, data)) if on_error is not None else None,
    )


def make_request_default(
    endpoint,
    method="GET",
    success_callback=None,
    failure_callback=None,
    progress_callback=None,
    redirect_callback=None,
    data=None,
    body=None,
    header=None,
    params=None,
):
    """
    Creates a URL Request with simplified, default callbacks. Error,
    failure, and redirect will be condensed into one callback.

    Args:
        endpoint (str): The endpoint the request will go to.

    Kwargs:
        method (str): The type of request being made. Defaults to 'GET'

        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(result (dict), data (dict))
        Defaults to None.

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(result (dict), data (dict))
        Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

        body (dict): Body of the request, will be encoded using
        urllib.urlencode. Defaults to None.

        header (dict): The header for the request. Defaults to None.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Each callback will receive the
        data in here. Defaults to empty dict.

    Return:
        UrlRequest: The request being made.

    """
    if data is None:
        data = {}
    data["success_callback"] = success_callback
    data["failure_callback"] = failure_callback
    data["progress_callback"] = progress_callback
    data["redirect_callback"] = redirect_callback
    return make_request(
        endpoint,
        method=method,
        on_success=default_success,
        on_failure=default_failure,
        on_error=default_error,
        on_redirect=default_redirect,
        on_progress=default_progress,
        body=body,
        header=header,
        data=data,
        params=params,
    )


def make_request_custom_success(
    endpoint,
    success_handler,
    method="GET",
    success_callback=None,
    failure_callback=None,
    redirect_callback=None,
    progress_callback=None,
    data=None,
    body=None,
    header=None,
    params=None,
):
    """
    Creates a request with a custom success handler and the default failure
    and progress handlers.

    Args:
        endpoint (str): The endpoint the request will go to.

        success_handler (function): Callback for a successful request,
        will have the signature. Typically used to modify the args being sent
        to the success_callback. Your success_handler should call the
        success_callback provided as a kwarg if available.
            on_success(request (UrlRequest), result (dict), data (dict))

    Kwargs:
        method (str): The type of request being made. Defaults to 'GET'

        success_callback (function): Callback for a successful request,
        will have a signature determined by the success_handler.
        Defaults to None.

        failure_callback (function): Callback for redirects, failures, and
        errors. Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'redirect', 'failure'. Defaults
        to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(result (dict), data (dict))
        Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

        body (dict): Body of the request, will be encoded using
        urllib.urlencode. Defaults to None.

        header (dict): The header for the request. Defaults to None.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Each callback will receive the
        data in here. Defaults to empty dict.

    Return:
        UrlRequest: The request being made.

    """
    if data is None:
        data = {}
    data["success_callback"] = success_callback
    data["failure_callback"] = failure_callback
    data["progress_callback"] = progress_callback
    data["redirect_callback"] = redirect_callback
    return make_request(
        endpoint,
        method=method,
        on_success=success_handler,
        on_failure=default_failure,
        on_error=default_error,
        on_redirect=default_redirect,
        on_progress=default_progress,
        body=body,
        header=header,
        data=data,
        params=params,
    )


def default_redirect(req, results, data):
    """
    Default handler for a redirect callback. Will call the 'redirect_callback'
    provided in data with the args: response headers (dict), data (dict)

    If the value of 'failure_callback' is None, nothing will be called.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'redirect_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["redirect_callback"] is None:
        return
    data["redirect_callback"](req, req._resp_headers, data)


def default_success(req, results, data):
    """
    Default handler for a success callback. Will call the 'failure_callback'
    provided in data with the args: results, data

    If the value of 'success_callback' is None, nothing will be called.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["success_callback"] is None:
        return
    data["success_callback"](results, data)


def default_failure(req, results, data):
    """
    Default handler for a failure callback. Will call the 'success_callback'
    provided in data with the args: 'failure', results, data

    If the value of 'failure_callback' is None, nothing will be called.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'failure_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["failure_callback"] is None:
        return
    data["failure_callback"]("failure", results, data)


def default_error(req, results, data):
    """
    Default handler for an error callback. Will call the 'failure_callback'
    provided in data with the args: 'error', results, data

    If the value of 'failure_callback' is None, nothing will be called.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'failure_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["failure_callback"] is None:
        return
    data["failure_callback"]("error", results, data)


def default_progress(req, current_size, total_size, data):
    """
    Default handler for a progress callback. Will call the 'progress_callback'
    provided in data with the args: current_size, total_size, data

    If the value of 'progress_callback' is None, nothing will be called.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'progress_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["progress_callback"] is None:
        return
    data["progress_callback"](current_size, total_size, data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class PodiumApplication:
    def __init__(self, app_id, app_secret, podium_url=None):
        self.app_id = app_id
        self.app_secret = app_secret
        self.podium_url = "https://podium.live" if podium_url is None else podium_url



#!/usr/bin/env python
# -*- coding: utf-8 -*-
class PodiumEvent(object):
    """
    Object that represents an Event.

    **Attributes:**
        **event_id** (str): Unique id for event

        **uri** (str): Endpoint for accessing full event information.

        **devices_uri** (str): Endpoint for accessing devices that
        attended the event.

        **title** (str): Title of Event

        **start_time** (str): Start time of the event. ISO 8601 format.

        **end_time** (str): End time of the event. ISO 8601 format.

        **venue_uri** (str): Endpoint for accessing venue information.

        **private** (bool): Is the event only viewable to creator?

        **user_uri** (str): Endpoint for accessing user information

        **user_avatar_url** (str): URL for user avatar
    """

    def __init__(
        self,
        event_id,
        uri,
        devices_uri,
        title,
        start_time,
        end_time,
        venue_uri,
        venue_id,
        private,
        user_uri,
        user_avatar_url,
    ):
        self.event_id = event_id
        self.uri = uri
        self.devices_uri = devices_uri
        self.title = title
        self.start_time = start_time
        self.end_time = end_time
        self.venue_uri = venue_uri
        self.venue_id = venue_id
        self.private = private
        self.user_uri = user_uri
        self.user_avatar_url = user_avatar_url


def get_event_from_json(json):
    """
    Returns a PodiumEvent object from the json dict received from podium api.

    Args:
        json (dict): Dict of data from REST api

    Return:
        PodiumEvent: The PodiumEvent object for this data.
    """
    return PodiumEvent(
        json["id"],
        json["URI"],
        json.get("devices_uri", None),
        json.get("title", None),
        json.get("start_time", None),
        json.get("end_time", None),
        json.get("venue_uri", None),
        json.get("venue_id", None),
        json.get("private", None),
        json.get("user_uri", None),
        json.get("user_avatar_url", None),
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
class PodiumPreset(object):
    """
    Object that represents a Preset.

    **Attributes:**
        **preset_id** (int): Unique id for preset

        **uri** (str): Endpoint for accessing full preset information.

        **name** (str): Name of the preset.

        **notes** (str): Notes for this preset

        **preset** (str): JSON data for preset

        **type** (int): mapping type key

        **mapping_type** (str): string key representation of the mapping type

        **updated** (str): Date preset was updated. ISO 8601 format.

        **created** (str): Date preset was created. ISO 8601 format.
    """

    MAPPING_TYPE_ANALOG = "analog"
    MAPPING_TYPE_CAN = "can"
    MAPPING_TYPE_OBDII = "obd2"
    MAPPING_TYPE_PODIUM_DASH = "podium_dash"
    MAPPING_TYPE_PODIUM_CONNECT = "PC"
    MAPPING_TYPE_RACECAPTURE_ANALYSIS = "rc_analysis"
    MAPPING_TYPE_RACECAPTURE_PRO = "RCP"
    MAPPING_TYPE_RACECAPTURE_TRACK = "RCT"
    MAPPING_TYPE_RACECAPTURE_DASH = "rc_dash"
    MAPPING_TYPE_SCRIPTING = "Script"

    def __init__(
        self,
        preset_id,
        uri,
        name,
        notes,
        preset_data,
        type,
        private,
        rating,
        rating_count,
        user_uri,
        preview_image_url,
        updated,
        created,
    ):
        self.preset_id = preset_id
        self.uri = uri
        self.name = name
        self.notes = notes
        self.preset_data = preset_data
        self.type = type
        self.private = private
        self.rating = rating
        self.rating_count = rating_count
        self.user_uri = user_uri
        self.preview_image_url = preview_image_url
        self.updated = updated
        self.created = created


def get_preset_from_json(json):
    """
    Returns a PodiumEvent object from the json dict received from podium api.

    Args:
        json (dict): Dict of data from REST api

    Return:
        PodiumEvent: The PodiumEvent object for this data.
    """
    return PodiumPreset(
        json["id"],
        json["URI"],
        json.get("name", None),
        json.get("notes", None),
        json.get("preset_data", None),
        json.get("type", None),
        json.get("private", None),
        json.get("rating", None),
        json.get("rating_count", None),
        json.get("user_uri", None),
        json.get("preview_image_url", None),
        json.get("created", None),
        json.get("updated", None),
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from podium_api.asyncreq import get_json_header_token, make_request_custom_success
from podium_api.types.user import get_user_from_json


def make_user_get(
    token,
    endpoint,
    expand=False,
    quiet=None,
    success_callback=None,
    failure_callback=None,
    progress_callback=None,
    redirect_callback=None,
):
    """
    Returns a PodiumUser object found at the uri provided in the endpoint
    arg.

    Args:
        token (PodiumToken): The authentication token for this session.

        endpoint (str): The URI to make the request to. Typically should be
        provided by some api object.

    Kwargs:
        expand (bool): Expand all objects in response output. Defaults to False

        quiet (object): If not None HTML layout will not render endpoint
        description. Defaults to None.

        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(token (string))
        Defaults to None.

        failure_callback (function): Callback for redirects, failures, and
        errors. Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'redirect', 'failure'. Defaults
        to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """
    params = {}
    if expand is not None:
        params["expand"] = expand
    if quiet is not None:
        params["quiet"] = quiet
    header = get_json_header_token(token)
    return make_request_custom_success(
        endpoint,
        users_success_handler,
        method="GET",
        success_callback=success_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        redirect_callback=redirect_callback,
        params=params,
        header=header,
    )


def users_success_handler(req, results, data):
    """
    Creates and returns a  PodiumToken to the success_callback found in data.

    Called automatically by **make_login_request**.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["success_callback"] is not None:
        data["success_callback"](get_user_from_json(results["user"]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
class PodiumAlertMessage(object):
    """
    Object that represents an Alert Message

    **Attributes:**

        **alertmessage_id** (int): Id of alertmessage

        **send_time** (str): Time the message was sent. ISO 8601 format.

        **ack_time** (str): Time the message was acknowledged. ISO 8601 format.

        **message** (str): Message of the alert

        **priority** (int): Priority level of the message

        **sender_id** (int): Id of the alertmessage sender

        **eventdevice_uri:** (str): URI of the eventdevice this alertmessage belongs to

        **device_uri:** (str): URI of the device this alertmessage belongs to

        **user_uri** (str): URI of the user this alertmessage belongs to
    """

    def __init__(
        self,
        alertmessage_id,
        uri,
        send_time,
        ack_time,
        message,
        priority,
        sender_id,
        eventdevice_uri,
        device_uri,
        user_uri,
    ):
        self.alertmessage_id = alertmessage_id
        self.uri = uri
        self.send_time = send_time
        self.ack_time = ack_time
        self.message = message
        self.priority = priority
        self.sender_id = sender_id
        self.eventdevice_uri = eventdevice_uri
        self.device_uri = device_uri
        self.user_uri = user_uri


def get_alertmessage_from_json(json):
    """
    Returns an AlertMessage object from the json dict received from
    podium api.

    Args:
        json (dict): Dict of data from REST api

    Return:
        AlertMessage: The AlertMessage object for the data.

    """
    return PodiumAlertMessage(
        json["id"],
        json["URI"],
        json["send_time"],
        json["ack_time"],
        json["message"],
        json["priority"],
        json["sender_id"],
        json["eventdevice_uri"],
        json["device_uri"],
        json["user_uri"],
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class PodiumVenue(object):
    """
    Object that represents a Venue.

    **Attributes:**
        **venue_id** (int): Venue Id

        **uri** (string): URI for the Venue.

        **name** (string): The Venue's name.
    """

    def __init__(
        self,
        venue_id,
        uri,
        events_uri,
        updated,
        created,
        name,
        centerpoint,
        country_code,
        configuration,
        track_map_array,
        start_finish,
        finish,
        sector_points,
        length,
    ):
        self.venue_id = venue_id
        self.uri = uri
        self.events_uri = events_uri
        self.updated = updated
        self.created = created
        self.name = name
        self.centerpoint = centerpoint
        self.country_code = country_code
        self.configuration = configuration
        self.track_map_array = track_map_array
        self.start_finish = start_finish
        self.finish = finish
        self.sector_points = sector_points
        self.length = length


def get_venue_from_json(json):
    """
    Returns a PodiumVenue object from the json dict received from podium api.

    Args:
        json (dict): Dict of data from REST api

    Return:
        PodiumVenue: The PodiumVenue object for the data.
    """
    return PodiumVenue(
        json["id"],
        json["URI"],
        json["events_uri"],
        json["updated"],
        json["created"],
        json.get("name", None),
        json.get("centerpoint", None),
        json.get("country_code", None),
        json.get("configuration", None),
        json.get("track_map_array", None),
        json.get("start_finish", None),
        json.get("finish", None),
        json.get("sector_points", None),
        json.get("length", None),
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import podium_api
from podium_api.asyncreq import get_json_header_token, make_request_custom_success
from podium_api.types.paged_response import get_paged_response_from_json
from podium_api.types.rating import get_rating_from_json
from podium_api.types.redirect import get_redirect_from_json


def make_rating_create(
    token,
    rateable_type,
    rateable_id,
    rating,
    report=None,
    success_callback=None,
    failure_callback=None,
    progress_callback=None,
    redirect_callback=None,
):
    """
    Request that creates or updates a rating for a rateable object.

    The uri for the newly created rating will be provided to the
    redirect_callback if one is provided in the form of a PodiumRedirect.

    Args:
        token (PodiumToken): The authentication token for this session.

        rateable_type (string): type of rateable object. e.g. preset

        rateable_id (int): ID of the object to rate

        rating (float): Rating value

    Kwargs:
        report (int): non zero if the rateable item is to be reported. rating must also be 0

        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(result (dict), data (dict))
        Defaults to None..

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(redirect_object (PodiumRedirect))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """
    endpoint = "{}/api/v1/{}/{}/ratings".format(podium_api.PODIUM_APP.podium_url, rateable_type.lower(), rateable_id)

    body = {"rating[rating]": rating}
    if report is not None:
        body["rating[report]"] = report
    header = get_json_header_token(token)
    return make_request_custom_success(
        endpoint,
        None,
        method="POST",
        success_callback=success_callback,
        redirect_callback=create_rating_redirect_handler,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        body=body,
        header=header,
        data={"_redirect_callback": redirect_callback},
    )


def create_rating_redirect_handler(req, results, data):
    """
    Handles the success redirect of a **make_rating_create** call.

    Returns a PodiumRedirect with a uri for the newly created rating to the
    _redirect_callback found in data.

    Automatically called by **make_rating_create**, will call the
    redirect_callback passed in to **make_rating_create** if there is on.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["_redirect_callback"] is not None:
        data["_redirect_callback"](get_redirect_from_json(results, "rating"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class PodiumUser(object):
    """
    Object that represents a particular User.

    **Attributes:**
        **user_id** (int): User id

        **uri** (string): URI for the User.

        **username** (string): The User's username.

        **description** (string): The User's description.

        **avatar_url** (string): User's avatar image url.

        **links** (list): 3rd party links for the user.

        **friendships_uri** (string): URI to friends list.

        **followers_uri** (string): URI to followers list.

        **friendship_uri** (string): If this User has been friended

        **events_uri** (string): URI to events for this user

        **venues_uri** (string): URI to venues this user particiated at

        by the user this attr will have a value, otherwise None. Defaults to
        None.
    """

    def __init__(
        self,
        user_id,
        uri,
        username,
        description,
        avatar_url,
        profile_image_url,
        links,
        friendships_uri,
        followers_uri,
        friendship_uri,
        events_uri,
        venues_uri,
    ):
        self.user_id = user_id
        self.uri = uri
        self.username = username
        self.description = description
        self.avatar_url = avatar_url
        self.profile_image_url = profile_image_url
        self.links = links
        self.friendships_uri = friendships_uri
        self.followers_uri = followers_uri
        self.friendship_uri = friendship_uri
        self.events_uri = events_uri
        self.venues_uri = venues_uri


def get_user_from_json(json):
    """
    Returns a PodiumUser object from the json dict received from podium api.

    Args:
        json (dict): Dict of data from REST api

    Return:
        PodiumUser: The PodiumUser object for the data.
    """
    return PodiumUser(
        json["id"],
        json["URI"],
        json["username"],
        json["description"],
        json["avatar_url"],
        json["profile_image_url"],
        json["links"],
        json["friendships_uri"],
        json["followers_uri"],
        json.get("friendship_uri", None),
        json["events_uri"],
        json["venues_uri"],
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
class PodiumLap(object):
    """
    Object that represents a Lap.

    **Attributes:**
        **uri** (str): Endpoint for accessing the lap.

         **raw_data_uri** (str): URI for accessing logged data.

         **lap_number** (str): Number for the lap.

         **end_time** (str): Time lap ended.

         **aggregates** (list): List of min/max/avg values for channels.

         **lap_time** (float): Lap time in minutes.

    """

    def __init__(self, uri, raw_data_uri, lap_number, end_time, aggregates, lap_time):
        self.uri = uri
        self.raw_data_uri = raw_data_uri
        self.lap_number = lap_number
        self.end_time = end_time
        self.aggregates = aggregates
        self.lap_time = lap_time


def get_lap_from_json(json):
    """
    Returns a PodiumLap object from the json dict received from podium api.

    Args:
        json (dict): Dict of data from REST api

    Return:
        PodiumEvent: The PodiumEvent object for this data.
    """
    return PodiumLap(
        json["URI"],
        json["raw_data_uri"],
        json["lap_number"],
        json["end_time"],
        json.get("aggregates", None),
        json["lap_time"],
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
class PodiumDevice(object):
    """
    Object that represents a Device.

    **Attributes:**
        **device_id** (str): Unique id for device.

        **uri** (str): Endpoint for accessing full device information.

        **serial** (str): Unique id for RaceCapture devices to identify
        themselves.

        **name** (str): Name of the device.

        **private** (bool): Is the event only viewable to creator?
    """

    def __init__(self, device_id, uri, serial, name, private):
        self.device_id = device_id
        self.uri = uri
        self.serial = serial
        self.name = name
        self.private = private


def get_device_from_json(json):
    """
    Returns a PodiumEvent object from the json dict received from podium api.

    Args:
        json (dict): Dict of data from REST api

    Return:
        PodiumEvent: The PodiumEvent object for this data.
    """
    return PodiumDevice(
        json["id"], json["URI"], json.get("serial", None), json.get("name", None), json.get("private", None)
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
class PodiumEventDevice(object):
    """
    Object that represents a device at an event.

    **Attributes:**
        **eventdevice_id** (str): Unique id for device at this event

        **uri** (str): URI for this device at this event.

        **channels** (list): List of channels of data. Can be sensors or
        other sources of data.

        **name** (str): Name of device at event. Not always the same as the
        device name.

        **device_uri** (str): URI of the device

        **laps_uri** (str): URI of lap data.

        **avatar_url** (str): URI of the device avatar

        **user_avatar_url** (str): URL of the user avatar

        **event_title** (str): Title of the event

    """

    def __init__(
        self,
        eventdevice_id,
        uri,
        channels,
        name,
        comp_number,
        device_uri,
        laps_uri,
        user_uri,
        event_uri,
        avatar_url,
        user_avatar_url,
        event_title,
        device_id,
        event_id,
    ):
        self.eventdevice_id = eventdevice_id
        self.uri = uri
        self.channels = channels
        self.name = name
        self.comp_number = comp_number
        self.device_uri = device_uri
        self.laps_uri = laps_uri
        self.user_uri = user_uri
        self.event_uri = event_uri
        self.avatar_url = avatar_url
        self.user_avatar_url = user_avatar_url
        self.event_title = event_title
        self.device_id = device_id
        self.event_id = event_id


def get_eventdevice_from_json(json):
    """
    Returns a PodiumEventDevice object from the json dict received from
    podium api.

    Args:
        json (dict): Dict of data from REST api

    Return:
        PodiumEvent: The PodiumEvent object for this data.
    """
    return PodiumEventDevice(
        json["id"],
        json["URI"],
        json.get("channels", []),
        json.get("name", None),
        json.get("comp_number", None),
        json.get("device_uri", None),
        json.get("laps_uri", None),
        json.get("user_uri", None),
        json.get("event_uri", None),
        json.get("avatar_url", None),
        json.get("user_avatar_url", None),
        json.get("event_title", None),
        json.get("device_id", None),
        json.get("event_id", None),
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class PodiumToken(object):
    """
    A PodiumToken holds the OAuth2 Token for a logged in user.

    **Attributes:**
        **token** (string): The OAuth2 token for the user.

        **token_type** (string): The type of token issued.

        **created** (int): The time created.register_podium_application.
    """

    def __init__(self, token, token_type, created):
        self.token = token
        self.token_type = token_type
        self.created = created


def get_token_from_json(json):
    """
    Returns a PodiumToken object from the json dict received from podium api.

    Args:
        json (dict): Dict of data from REST api

    Return:
        PodiumToken: The PodiumToken object for the data.
    """
    return PodiumToken(json["access_token"], json["token_type"], json["created_at"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from podium_api.asyncreq import get_json_header_token, make_request_custom_success
from podium_api.types.lap import get_lap_from_json
from podium_api.types.paged_response import get_paged_response_from_json


def make_lap_get(
    token,
    endpoint,
    expand=True,
    quiet=None,
    success_callback=None,
    redirect_callback=None,
    failure_callback=None,
    progress_callback=None,
):

    """
    Request that returns a PodiumLap that represents a specific
    lap found at the URI.

    Args:
        token (PodiumToken): The authentication token for this session.

        endpoint (str): The URI for the lap.

    Kwargs:
        expand (bool): Expand all objects in response output. Defaults to True

        quiet (object): If not None HTML layout will not render endpoint
        description. Defaults to None.

        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(PodiumLap)
        Defaults to None.

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(result (dict), data (dict))
        Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

    Return:
        UrlRequest: The request being made.

    """
    params = {}
    if expand is not None:
        params["expand"] = expand
    if quiet is not None:
        params["quiet"] = quiet
    header = get_json_header_token(token)
    return make_request_custom_success(
        endpoint,
        lap_success_handler,
        method="GET",
        success_callback=success_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        redirect_callback=redirect_callback,
        params=params,
        header=header,
    )


def make_laps_get(
    token,
    endpoint,
    start=None,
    per_page=None,
    expand=True,
    quiet=None,
    success_callback=None,
    redirect_callback=None,
    failure_callback=None,
    progress_callback=None,
):
    """
    Request that returns a PodiumPagedRequest of laps.

    Args:
        token (PodiumToken): The authentication token for this session.

        endpoint (str): The endpoint to make the request too.

    Kwargs:
        expand (bool): Expand all objects in response output. Defaults to True

        quiet (object): If not None HTML layout will not render endpoint
        description. Defaults to None.

        success_callback (function): Callback for a successful request,
        will have the signature:
            on_success(PodiumPagedResponse)
        Defaults to None.

        failure_callback (function): Callback for failures and errors.
        Will have the signature:
            on_failure(failure_type (string), result (dict), data (dict))
        Values for failure type are: 'error', 'failure'. Defaults to None.

        redirect_callback (function): Callback for redirect,
        Will have the signature:
            on_redirect(result (dict), data (dict))
        Defaults to None.

        progress_callback (function): Callback for progress updates,
        will have the signature:
            on_progress(current_size (int), total_size (int), data (dict))
        Defaults to None.

        start (int): Starting index for events list. 0 indexed.

        per_page (int): Number per page of results, max of 100.

    Return:
        UrlRequest: The request being made.

    """
    params = {}
    if expand is not None:
        params["expand"] = expand
    if quiet is not None:
        params["quiet"] = quiet
    if start is not None:
        params["start"] = start
    if per_page is not None:
        per_page = min(per_page, 100)
        params["per_page"] = per_page

    header = get_json_header_token(token)
    return make_request_custom_success(
        endpoint,
        laps_success_handler,
        method="GET",
        success_callback=success_callback,
        failure_callback=failure_callback,
        progress_callback=progress_callback,
        redirect_callback=redirect_callback,
        params=params,
        header=header,
    )


def laps_success_handler(req, results, data):
    """
    Creates and returns a PodiumPagedResponse with PodiumLap as the
    payload to the success_callback found in data if there is one.

    Called automatically by **make_laps_get**.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["success_callback"] is not None:
        data["success_callback"](get_paged_response_from_json(results, "laps"))


def lap_success_handler(req, results, data):
    """
    Creates and returns a PodiumLap.
    Called automatically by **make_lap_get**.

    Args:
        req (UrlRequest): Instace of the request that was made.

        results (dict): Dict returned by the request.

        data (dict): Wildcard dict for containing data that needs to be passed
        to the various callbacks of a request. Will contain at least a
        'success_callback' key.

    Return:
        None, this function instead calls a callback.

    """
    if data["success_callback"] is not None:
        data["success_callback"](get_lap_from_json(results["lap"]))



from distutils.core import setup

setup(
    name="podium_api",
    packages=["podium_api", "podium_api.types"],
    version="0.0.21",
    description="API for Podium Motorsports Platform http://podium.live",
    author="Autosport Labs",
    author_email="sales@autosportlabs.com",
    url="https://github.com/autosportlabs/podium-api",
    download_url="https://github.com/autosportlabs/podium-api/archive/0.0.21.tar.gz",
    keywords=["motorsports", "telemetry", "live streaming"],
    classifiers=[],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class PodiumAccount(object):
    """
    Object that represents a particular User.

    **Attributes:**
        **account_id** (str): Unique id for account

        **username** (string): The User's username.

        **email** (str): Account email address

        **user_uri** (str): URI to user associated with account

        **events_uri** (str): URI to account's events

        **devices_uri** (str): URI to account's devices

        **events_uri** (str): URI to account's events

        **streams_uri** (str): URI to account's current live streams

        **exports_uri** (str): URI to account's telemetry exports
    """

    def __init__(self, account_id, username, email, devices_uri, exports_uri, streams_uri, user_uri, events_uri):
        self.account_id = account_id
        self.username = username
        self.email = email
        self.devices_uri = devices_uri
        self.exports_uri = exports_uri
        self.streams_uri = streams_uri
        self.user_uri = user_uri
        self.events_uri = events_uri


def get_account_from_json(json):
    """
    Returns a PodiumAccount object from the json dict received from podium api.

    Args:
        json (dict): Dict of data from REST api

    Return:
        PodiumUser: The PodiumAccount object for the data.
    """
    return PodiumAccount(
        json["id"],
        json["username"],
        json["email"],
        json["devices_uri"],
        json["exports_uri"],
        json["streams_uri"],
        json["user_uri"],
        json["events_uri"],
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from podium_api.types.alertmessage import get_alertmessage_from_json
from podium_api.types.device import get_device_from_json
from podium_api.types.event import get_event_from_json
from podium_api.types.eventdevice import get_eventdevice_from_json
from podium_api.types.friendship import get_friendship_from_json
from podium_api.types.lap import get_lap_from_json
from podium_api.types.preset import get_preset_from_json
from podium_api.types.user import get_user_from_json
from podium_api.types.venue import get_venue_from_json


class PodiumPagedResponse(object):
    """
    Object that represents data returned from a paged request.

    **Attributes:**
        **payload** (list): The data returned for this page.

        **total** (int): The total number of events found.

        **next_uri** (str): The URI for next page of results if available

        **prev_uri** (str): The URI for previous page of results if available

        **payload_name** (str): If a payload_name is provided attribute access
        on this name will also return the payload. Defaults to None. Used to
        mirror the web api.
    """

    def __init__(self, payload, total, next_uri, prev_uri, payload_name=None):
        self.payload = payload
        self.total = total
        self.next_uri = next_uri
        self.prev_uri = prev_uri
        self.payload_name = payload_name

    def __getattr__(self, name):
        if name == self.payload_name:
            return self.payload
        else:
            raise AttributeError()


PAYLOAD_NAME_TO_OBJECT = {
    "events": get_event_from_json,
    "friendships": get_friendship_from_json,
    "users": get_user_from_json,
    "eventdevices": get_eventdevice_from_json,
    "devices": get_device_from_json,
    "laps": get_lap_from_json,
    "venues": get_venue_from_json,
    "alertmessages": get_alertmessage_from_json,
    "presets": get_preset_from_json,
}


def get_paged_response_from_json(json, payload_name):
    """
    Returns a PodiumPagedResponse object from the json dict received from
    podium api.

    Args:
        json (dict): Dict of data from REST api

        payload_name (str): Name of the actual paged data in the json dict.
        Will be used to determine the object the data gets converted into.
        The "payload" attr will also be returned with lookup by payload_name.

    Return:
        PodiumPagedResponse: The PodiumPagedResponse object for the data.
    """
    conversion_func = PAYLOAD_NAME_TO_OBJECT[payload_name]
    data = [conversion_func(x) for x in json[payload_name]]
    return PodiumPagedResponse(
        data, json["total"], json.get("nextURI", None), json.get("prevURI", None), payload_name=payload_name
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from podium_api.types.application import PodiumApplication

"""
The podium_api module allows you to asynchronously interact with the Podium
API. It is built on top of Kivy's UrlRequest. 

**Module Attributes:**
"""

"""

    **PODIUM_APP** (PodiumApplication): The instance of a PodiumApplication
    storing the app uuid and secret for the Podium API. Starts out as None,
    your program must call **register_podium_application** with the uuid and
    app secret before making any calls to podium_api.
    
"""

PODIUM_APP = None


def register_podium_application(app_id, app_secret, podium_url=None):
    """Registers an id and secret for the application for use with the Podium
    API. Should only ever be invoked once per run of program.

    Args:
        app_id (string): The UUID for your application as registered with
        the Podium API.

        app_secret (string): The secret for your application as registered with
        the Podium API.

        podium_url (String): Optional podium url. defaults to https://podium.live
    """

    global PODIUM_APP
    PODIUM_APP = PodiumApplication(app_id, app_secret, podium_url=podium_url)


def unregister_podium_application():
    global PODIUM_APP
    PODIUM_APP = None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
class PodiumRedirect(object):
    """
    Object representing the redirect return from the podium api.

    **Attributes:**
        **location** (str): URI for the redirected object.

        **type** (str): Type of object. Can be 'event', 'device', 'eventdevice'
    """

    def __init__(self, location, object_type):
        self.location = location
        self.object_type = object_type


def get_redirect_from_json(json, object_type):
    """
    Returns a PodiumRedirect object from the provided json dict.

    Args:
        json (dict): Dict of data from REST api

        object_type (str): The type of object this redirect URI represents.

    Return:
        PodiumRedirect: The PodiumRedirect object for this data.
    """

    # between development and production Location may be lower case location.
    # So, account for both
    try:
        location = json["location"]
    except KeyError:
        location = json["Location"]
    return PodiumRedirect(location, object_type)
